import express from "express";
import http from "http";
import { Server as IOServer } from "socket.io";
import queue from "./queue.js";
import path from "path";
import { fileURLToPath } from "url";

const PORT = 3000;
const app = express();
const server = http.createServer(app);
const io = new IOServer(server, {
    cors: {
        origin: "http://localhost:5173",
    },
});

const __filename = fileURLToPath(import.meta.url);
const __dirname = path.dirname(__filename);
const outputDir = path.join(__dirname, "../dist");

app.use(express.static(outputDir));

app.get("/*", function(req, res) {
    res.sendFile(path.join(outputDir, "index.html"));
});

const headers = {};

const socketIdList = [];

(async () => {
    // await queue.loadTracks("tracks");
    // queue.play();

    io.on("connection", (socket) => {
        // Every new streamer must receive the header

        socket.leave(socket.id);

        socket.on("disconnect", () => {
            var index = socketIdList.indexOf(socket.id);
            if (index !== -1) {
                socketIdList.splice(index, 1);
            }
            console.log("SocketId List: ", socketIdList)
        });

        if (queue.bufferHeader) {
            socket.emit("bufferHeader", queue.bufferHeader);
        }

        socket.on("createRoom", () => {
            // console.log("Create")
            socket.join(socket.id)
            // console.log(io.sockets.adapter.rooms)
        });

        socket.on("joinRoom", () => {
            const currentRooms = io.sockets.adapter.rooms
            console.log("Join")
            // console.log(socket.id)
            // console.log(currentRooms)
            console.log("Socket List: ", socketIdList)
            currentRooms.forEach((value,key) => {
                if (!socketIdList.includes(key) && !socketIdList.includes(socket.id)) {
                    // socket.join(key)
                    socket.join(socket.id);
                    // console.log(socket.id)
                    socketIdList.push(key)
                    socketIdList.push(socket.id)
                    socket.to(key).emit("newJoin", socket.id)
                }
                // socket.join(socket.id);
                // console.log(socket.id, key, io.sockets.adapter.rooms, headers)
                // socket.emit("bufferHeader", headers[key]);
                // socket.leave(socket.id);
                // console.log(io.sockets.adapter.rooms.get(key).size)
            })
        });

        socket.on("bufferHeader", (header, socketId) => {
            headers[socket.id] = header
            // queue.bufferHeader = header;
            // socket.broadcast.emit("bufferHeader", queue.bufferHeader);
            // console.log(222, socketId)
            socket.to(socketId).emit("bufferHeader", header);
            // socket.broadcast.emit("bufferHeader", header);
            // socket.broadcast.to(socket.id).emit("bufferHeader", header);
        });

        socket.on("stream", (packet, socketId) => {
            // Only broadcast microphone if a header has been received
            // if (!queue.bufferHeader) return;
            // console.log(packet)
            // Audio stream from host microphone
            // socket.broadcast.emit("bufferHeader", queue.bufferHeader);
            // console.log(999)
            // console.log(socketId)
            socket.to(socketId).emit("stream", packet);
            // socket.broadcast.emit("stream", packet);
            // socket.broadcast.to(socket.id).emit("stream", packet);
        });

        // socket.on("control", (command) => {
        //     switch (command) {
        //         case "pause":
        //             queue.pause();
        //             break;
        //         case "resume":
        //             queue.resume();
        //             break;
        //     }
        // });
    });

    // HTTP stream for music
    // app.get("/stream", (req, res) => {
    //     const { id, client } = queue.addClient();

    //     res.set({
    //         "Content-Type": "audio/mp3",
    //         "Transfer-Encoding": "chunked",
    //     }).status(200);

    //     client.pipe(res);

    //     req.on("close", () => {
    //         queue.removeClient(id);
    //     });
    // });

    server.listen(PORT, () => {
        console.log(`Listening on port ${PORT}`);
    });
})();

export { };
